<?php

namespace Drupal\oswald\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Class ConfigForm.
 */
class ConfigForm extends ConfigFormBase {

  use MessengerTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  private $entityTypeManager;

  /**
   * SynonymForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $entityTypeManager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'oswald_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $contentTypes = $this->entityTypeManager->getStorage('node_type')
      ->loadMultiple();
    $contentTypesList = [];

    // List all content-types available.
    foreach ($contentTypes as $contentType) {
      $contentTypesList[$contentType->id()] = $contentType->label();
    }

    $form['oswald_test'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Oswald test'),
    ];

    $admin_link = Link::fromTextAndUrl(t('here'), Url::fromRoute('oswald.config_test'))->toString();
    $form['oswald_test']['testing_info'] = [
      '#type' => 'markup',
      '#markup' => $this->t(' Test your installation @here.', ['@here' => $admin_link]),
    ];

    $form['oswald_general'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Oswald general'),
    ];
    $form['oswald_general']['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Oswald unique key'),
      '#description' => $this->t('The Key from your oswald platform.'),
      '#default_value' => $this->config('oswald.general_settings')
        ->get('key'),
    ];
    $form['oswald_general']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Oswald chatbot integration.'),
      '#description' => $this->t('Enable oswald for this website.'),
      '#default_value' => $this->config('oswald.general_settings')
        ->get('enable'),
    ];

    $oswaldTypeList = [
      'site_wide' => $this->t('Site wide'),
      //'selected_only' => $this->t('Selected paths only'),
      //'selected_not' => $this->t('Everywhere except selected paths'),
    ];

    $form['oswald_general']['embed_type'] = [
      '#type' => 'select',
      '#multiple' => FALSE,
      '#title' => $this->t('Where to enable Oswald'),
      '#options' => $oswaldTypeList,
      '#description' => $this->t('Site wide means everywhere except on admin pages (admin theme).'),
      '#default_value' => $this->config('oswald.general_settings')
        ->get('embed_type'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('oswald.general_settings')
      ->set('key', $form_state->getValue('key'))
      ->set('enable', $form_state->getValue('enable'))
      ->set('embed_type', $form_state->getValue('embed_type'))
      //->set('content_type_filters', $form_state->getValue('content_type_filters'))
      ->save();

    if ($form_state->getValue('key')) {
      $this->messenger()
        ->addStatus($this->t('Oswald configuration saved successfully. Your bot is now Active!'));
    }
    else {
      $this->messenger()
        ->addStatus($this->t('Oswald configuration saved successfully.'));
    }

  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['oswald.general_settings'];
  }

}
